@@ -33,7 +33,8 @@
 #include <CGAL/QP_solver/assertions.h>
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
-#include <boost/iterator/counting_iterator.hpp>
+
+#include <CGAL/boost/iterator/counting_iterator.hpp>
 #include <boost/iterator/transform_iterator.hpp>
 
 namespace CGAL {
